;;;; This program turns Markdown documents into code by converting
;;;; documentation lines into comments and code lines into code.
;;;;
;;;; It is invoked from the command-line as follows:
;;;;
;;;; > cat file.md | path/to/litboot.fnl <line-comment-word>
;;;;
;;;; The result is printed on stdout.
;;;;
;;;; It's assumed that all the code is written in one language. The first
;;;; argument of this script is the word used to start a line comment.
(local line-comment-word (. arg 1))

(when (or (not line-comment-word)
          (= "-h" line-comment-word)
          (= "--help" line-comment-word))
  (print (.. "Usage: " (. arg 0) " <line-comment-word>\n"
             "See " (. arg 0) " for the manual."))
  (os.exit))
;;;;
;;;; It's also assumed that code is placed in fenced code blocks. Where the
;;;; fences consist of backticks and occur at the start of the line.
;;;;
;;;; The algorithm is simple:
;;;; - Lines inside fenced code blocks are considered code.
;;;; - Lines outside fenced code blocks are considered documentation.
;;;; - Documentation lines are printed with the line-comment word as a prefix.
;;;; - Code lines are printed as is.
;;;; - Fences are discarded.
;;;;
(var line-is-documentation-line true)

(each [line (io.lines)]
  (if (= (string.sub line 1 3)
         "```")
    (set line-is-documentation-line (not line-is-documentation-line))
    (print (if line-is-documentation-line
             (.. line-comment-word line)
             line))))
